@@ -6,8 +6,13 @@
 checkDependencies <- function(method){
   if(method %in% c("ICP", "hiddenICP")){
     missingDependenciesMessage("InvariantCausalPrediction", method)
-  }else if(method %in% c("pc", "LINGAM", "ges", "gies", "rfci")){
+  }else if(method %in% c("rankPc", "pc", "LINGAM", "rankArges",
+                         "arges", "rankGes", "ges", "gies", "rankGies",
+                         "rankFci", "fci", "rfci", "fciplus")){
     missingDependenciesMessage("pcalg", method)
+  }else if(method %in% c("rankArges", "arges")){
+    missingDependenciesMessage("huge", method)
+    missingDependenciesMessage("flare", method)
   }else if(method == "CAM"){
     missingDependenciesMessage("CAM", method)
   }else if(method == "regression"){
@@ -17,6 +22,10 @@ checkDependencies <- function(method){
   }else if(method == "bivariateANM"){
     missingDependenciesMessage("kernlab", method)
     missingDependenciesMessage("mgcv", method)
+  }else if(method == "mmhc"){
+    missingDependenciesMessage("bnlearn", method)
+  }else if(method == "directLINGAM"){
+    missingDependenciesMessage("R.matlab", method)
   }else{
     if(method != "backShift")
       stop(paste("Method", method, "not (yet?) implemented."))
